$ Title Hybrid Top-down Computable General Equilibrium and Bottom-up Electricity Expansion Model
$ontext
--------------------------------------------------------------------------------
------------------------------ General Header ----------------------------------
--------------------------------------------------------------------------------
         Hybrid Top-down Computable General Equilibrium and Bottom-up Electricity Expansion Model
         Renato Rodrigues
         Instituto de Investigacin Tecnolgica
         Date: 13, Sep, 2011
--------------------------------------------------------------------------------
--------------------------- End of General Header ------------------------------
--------------------------------------------------------------------------------
$offtext

*-------------------------------------------------------------------------------
*------------------------------------- Options ---------------------------------
*-------------------------------------------------------------------------------
$include ./Auxiliar_Files/options.gms

*Maximum time available to solve in seconds. The default value is 1000.
Option Reslim=100000;

*-------------------------------------------------------------------------------
*-------------------------------------- Sets -----------------------------------
*-------------------------------------------------------------------------------
*Including macros
$include ./Auxiliar_Files/macros.gms

*Defining sets
$include ./Auxiliar_Files/sets.gms

*-------------------------------------------------------------------------------
*----------------------------------- Parameters --------------------------------
*-------------------------------------------------------------------------------
$if not set excel_cell_separator $set excel_cell_separator ,
*set it to "," if you are using an english excel or ";" for an spanish excel


*Defining Exogenous Parameters
$include ./Auxiliar_Files/exogenous_parameters.gms

*-------------------------------------------------------------------------------
*---------------------------- Default Command Line Options ---------------------
*-------------------------------------------------------------------------------
*You can change these options unless expressed otherwise

*Export options
* 1 = ON / 0 = OFF
* Enable export results to excel files
$if not set excel_export $set excel_export 1
* Enable export results to GDX files
$if not set gdx_export $set gdx_export 1
* Enable GNU PLOT
$if not set gnu_plot $set gnu_plot 0
*Choose the model to be executed:
$if not set Model $set Model TAX_HGEMED


*-----------Simulation Options

*SOx - any number - 0 (to not use) - recomended: 1000 /t or 2000 /t
$if not set SOX_tax $set SOX_tax 1000

*NOx - any number - 0 (to not use) - recomended: 1000 /t or 2000 /t
$if not set NOX_tax $set NOX_tax 1000

*CO2e Tax Level - any number - 0 (to not use) - recomended: 10 /tCO2e or 30 /tCO2e
$if not set CO2e_diffuse_sectors_tax $set CO2e_diffuse_sectors_tax 0

* Set to 1 if you want to acquire the renewable premium money through taxes on final products applied to the electricity sector only, 2 if applied to energy sectors, 3 if applied to all sectors -  0 (to not use)
$if not set renewable_premium_tax $set renewable_premium_tax 0

* set to: 0 to leave the environmental tax income to the public administration, 1 to apply it on subsidies to diminishes social contributions taxes or 2 to apply it on lump sum transfers to the productive sectors
$if not set green_tax_reform $set green_tax_reform 0

* set to: 0, 1 or 2
$if not set fuel_tax_simulation $set fuel_tax_simulation  0

*Post-Processing simulations
$if %renewable_premium_tax%==0 $Set renewable_premium_tax_incidence_sectors Electricity
$if %renewable_premium_tax%==1 $Set renewable_premium_tax_incidence_sectors Electricity
$if %renewable_premium_tax%==2 $Set renewable_premium_tax_incidence_sectors Oil-Nuclear, Gas, Electricity
$if %renewable_premium_tax%==3 $Set renewable_premium_tax_incidence_sectors Manufactures, Coal, Oil-Nuclear, Gas, Electricity, Transport, Other_Services
$if %renewable_premium_tax%==4 $Set renewable_premium_tax_incidence_sectors Electricity
$if %renewable_premium_tax%==5 $Set renewable_premium_tax_incidence_sectors Oil-Nuclear, Gas, Electricity
$if %renewable_premium_tax%==6 $Set renewable_premium_tax_incidence_sectors Manufactures, Coal, Oil-Nuclear, Gas, Electricity, Transport, Other_Services
set renewable_premium_tax_sectors(s) / %renewable_premium_tax_incidence_sectors% /;

*-------------------------------------------------------------------------------
*-------------------------- File Name and Report Variables ---------------------
*-------------------------------------------------------------------------------
* These variables are only used for setting file names and reporting logs. Their values are automatically set by the GEMED.exe application.
*Load block aggregation
$Set Load_Block_Aggregation LB_1
*Choose the calibration aproximation method
$Set Calibration_Problem MinMaxPlus
*Choose the years to be simulated in the electricity operation and investment model
$Set Years_Simulated 2005-2005

*-------------------------------------------------------------------------------
*-------------------------------------- Report Code ----------------------------
*-------------------------------------------------------------------------------

set model_description /
*        THESIS MODELS
         BU                              Bottom-up Electricity Model
         CGE                             Top-down Pure CGE Model
         GEMED                           GEMED - Electricity Extended Top-down Pure CGE Model
         HGEMED                          H-GEMED - Hybrid Top-down CGE Bottom-up Electricity Model
*        AUXILIAR MODELS
         Aux_SAM_Calibration             SAM disaggregation in load blocks and technologies
         Aux_SAM_Iter_Calibration        Iterative SAM disaggregation and electricity model results compatibility
         Aux_BU_Comparison               Bottom-up Electricity Lineal and MCP Results Comparison
*        POLICY EVALUATION - Active Demand Response Management policies
         DR_BU                           Demand Response in the pure Bottom-up Electricity Expansion Lineal Model
         DR_CGE                          Demand Response in the pure CGE Model
         DR_GEMED                        Demand Response in the GEMED Model with single SAM calibration
         DR_GEMED_Iter_Calibration       Demand Response in the GEMED Model with iterative calibration
         DR_HGEMED                       Demand Response in the H-GEMED Model
*        POLICY EVALUATION - Tax policies
         TAX_BU                          Tax in the pure Bottom-up Electricity Expansion Lineal Model
         TAX_CGE                         Tax in the pure CGE Model
         TAX_GEMED                       Tax in the GEMED Model with single SAM calibration
         TAX_HGEMED                      Tax in the H-GEMED Model with iterative SAM calibration
/;

$if %Calibration_Problem%==MinMax $Set Abreviated_calibration_problem MM
$if %Calibration_Problem%==MinMaxPlus $Set Abreviated_calibration_problem MMP
$if %Calibration_Problem%==Quadratic $Set Abreviated_calibration_problem Q

*Creating resumed model execution log file (Executions_log.txt)
File Execution_log /Execution.log/;
put Execution_log;
*Executions_log.ap = 1;
put model_description.te("%Model%") //;
put @5 'Execution date and time...............      'system.DATE ' ' system.TIME  /;
put @5 'Model Code............................      %Model%' /;
put @5 'Load block aggregation used...........      %Load_Block_Aggregation%' /;
put @5 'Calibration problem objective.........      %Calibration_Problem%' /;
put @5 'Years simulated.......................      %Years_Simulated%' /;
put @5 'SOX tax...............................      %SOX_tax% /tSOx' /;
put @5 'NOX tax...............................      %NOX_tax% /tNOx' /;
put @5 'CO2e_diffuse_sectors_tax..............      %CO2e_diffuse_sectors_tax% /tCO2e' /;
put @5 'renewable_premium_tax.................      %renewable_premium_tax% (1=ON)' /;
put @5 'green_tax_reform......................      %green_tax_reform% (1=Social Contributions, 2=Sectors subsidy)' //;

*-------------------------------------------------------------------------------
*---------------------------------- THESIS MODELS ------------------------------
*-------------------------------------------------------------------------------
$If "%Model%" == "BU" $goto Model_BU
$If "%Model%" == "CGE" $goto Model_CGE
$If "%Model%" == "GEMED" $goto Model_GEMED
$If "%Model%" == "HGEMED" $goto Model_HGEMED
$If "%Model%" == "Aux_SAM_Calibration" $goto Model_Aux_SAM_Calibration
$If "%Model%" == "Aux_SAM_Iter_Calibration" $goto Model_Aux_SAM_Iter_Calibration
$If "%Model%" == "Aux_BU_Comparison" $goto Model_Aux_BU_Comparison

$If "%Model%" == "DR_BU" $goto Model_DR_BU
$If "%Model%" == "DR_CGE" $goto Model_DR_CGE
$If "%Model%" == "DR_GEMED" $goto Model_DR_GEMED
$If "%Model%" == "DR_GEMED_Iter_Calibration" $goto Model_DR_GEMED_Iter_Calibration
$If "%Model%" == "DR_HGEMED" $goto Model_DR_HGEMED

$If "%Model%" == "TAX_BU" $goto Model_TAX_BU
$If "%Model%" == "TAX_CGE" $goto Model_TAX_CGE
$If "%Model%" == "TAX_GEMED" $goto Model_TAX_GEMED
$If "%Model%" == "TAX_HGEMED" $goto Model_TAX_HGEMED


*-------------------------------------------------------------------------------
*---------------------- BU = Bottom-up Electricity Model -----------------------
*-------------------------------------------------------------------------------
$label Model_BU
*-----------ELECTRICITY MODEL EXECUTION
$include ./Models/BU - Bottom-up Electricity Model.gms

*fixing pumping solution because strange things are happening on the mcp model
*PPUMPED_LEGEN.fx(eys,l,p,b)= 0;

* lineal model solution
         if (exec_BU_LP=1, SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU));
* start point solution
         if (exec_BU_MCP=1, SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU));

* exporting results
         if ( (exec_BU_LP=1) and (%gdx_export%=1), execute_unload 'LEGEN_BAU.gdx');

*fixing pumping solution because strange things are happening on the mcp model
*if (exec_BU_MCP=1, PPUMPED_LEGEN.fx(eys,l,p,b)=PPUMPED_LEGEN.l(eys,l,p,b); );

* mcp model solution
         if (exec_BU_MCP=1, SOLVE MCPGEN_BAU using MCP; execution_log_file(MCPGEN_BAU));

* exporting results
         if ( (%gdx_export%=1) and (exec_BU_MCP=1), execute_unload 'MCPGEN_BAU.gdx');


* Returning to Calibration Model if BU model was called from there
$If "%Model%" == "Aux_SAM_Calibration" $goto Model_BU_Return_to_Aux_SAM_Calibration
$If "%Model%" == "GEMED" $goto Model_BU_Return_to_Aux_SAM_Calibration
$If "%Model%" == "TAX_GEMED" $goto Model_BU_Return_to_Aux_SAM_Calibration

$goto end
*-------------------------------------------------------------------------------
*-------------------------- CGE = Top-down Pure CGE Model ----------------------
*-------------------------------------------------------------------------------
$label Model_CGE

*-----------CGE MODEL EXECUTION
$include ./Models/CGE - Top-down Pure CGE Model.gms
         if (exec_MCP=1, SOLVE CGE_MCP using MCP; execution_log_file(CGE_MCP));
         if (exec_NLP=1, SOLVE CGE_NLP minimizing DUMMY_CGE using nlp; execution_log_file(CGE_NLP));

*        returning parameters and variables to original non scaled values
CGE_scale_parameters(*)
CGE_scale_results(*)

* exporting results
         if ( (%gdx_export%=1) and (exec_MCP=1), execute_unload 'CGE_MCP_BAU.gdx');
         if ( (%gdx_export%=1) and (exec_NLP=1), execute_unload 'CGE_NLP_BAU.gdx');

$goto end
*-------------------------------------------------------------------------------
*------------ GEMED - Electricity Extended Top-down Pure CGE Model -------------
*-------------------------------------------------------------------------------
$label Model_GEMED

*-----------SAM CALIBRATION MODEL EXECUTION
$goto Model_Aux_SAM_Calibration
$label Model_Aux_SAM_Calibration_Return_to_GEMED


*        sending optimization results as parameters to the following model
         Send_Results_SAMDIS_to_GEMED

$ontext
parameter tempPrice(ey,l,month,day,hour,p,b);
         loop ((ey,l),
                 loop(p_month_day_hour(p,month,day,hour),
                         tempPrice(ey,l,month,day,hour,p,b)=(GAD_energy_only_price(ey,l,p,b))$(  ((rank(p,ey,l,month,day,hour) le (100-Block_min_percentil(b))*(maxrank(l,p)/100))) and ((rank(p,ey,l,month,day,hour) > (100-Block_max_percentil(b))*(maxrank(l,p)/100))) );
                 );
         );
display tempPrice;
$offtext

*-----------EXTENDEND CGE MODEL EXECUTION
$include ./Models/GEMED - Electricity Extended Top-down Pure CGE Model.gms
         if (exec_MCP=1, SOLVE GEMED_MCP using MCP; execution_log_file(GEMED_MCP));
         if (exec_NLP=1, SOLVE GEMED_NLP minimizing DUMMY_GEMED using nlp; execution_log_file(GEMED_NLP));

*        returning parameters and variables to original non scaled values
GEMED_scale_parameters(*)
GEMED_scale_results(*)

*        exporting results
         if ( (%gdx_export%=1) and (exec_MCP=1), execute_unload 'GEMED_MCP_BAU.gdx');
         if ( (%gdx_export%=1) and (exec_NLP=1), execute_unload 'GEMED_NLP_BAU.gdx');

$goto end
*-------------------------------------------------------------------------------
*------ HGEMED - H-GEMED - Hybrid Top-down CGE Bottom-up Electricity Model -----
*-------------------------------------------------------------------------------
$label Model_HGEMED

$set gnu_plot 0

*Forcing BU model to be solved as MCP to provide the initial solution to the HGEMED model
exec_BU_MCP=1;
exec_BU_LP=0;

*-----------ITERATIVE CALIBRATION MODEL EXECUTION
$goto Model_Aux_SAM_Iter_Calibration
$label Model_Aux_SAM_Iter_Calibration_Return_HGEMED

*-----------EXECUTING MCP BU MODEL TO SERVE AS STARTING POINT
         SOLVE MCPGEN_BAU using MCP;

*-----------H-GEMED MODEL EXECUTION
$include ./Models/H-GEMED - Hybrid Top-down CGE Bottom-up Electricity Model.gms

*PPUMPED_HGEMED.fx(eys,l,p,b)=PPUMPED_LEGEN.l(eys,l,p,b);

         if (exec_MCP=1, SOLVE HGEMED_MCP using MCP; execution_log_file(HGEMED_MCP));
         if (exec_NLP=1, SOLVE HGEMED_NLP minimizing DUMMY_HGEMED using nlp; execution_log_file(HGEMED_NLP));

*        returning parameters and variables to original non scaled values
         HGEMED_scale_parameters(*)
         HGEMED_scale_results(*)

*        exporting results
*$if %excel_export%==1 $include ./Auxiliar_Output_Files/HGEMED Output.gms

*        exporting results
         if ( (%gdx_export%=1) and (exec_MCP=1), execute_unload 'HGEMED_MCP_BAU.gdx');
         if ( (%gdx_export%=1) and (exec_NLP=1), execute_unload 'HGEMED_NLP_BAU.gdx');

* returning to model h-gemed tax model if this label was executed from there
$If "%Model%" == "TAX_HGEMED" $goto Model_Aux_SAM_Iter_Calibration_Return_TAX_HGEMED
$If "%Model%" == "DR_HGEMED" $goto Model_Aux_SAM_Iter_Calibration_Return_DR_HGEMED

$goto end
*-------------------------------------------------------------------------------







*-------------------------------------------------------------------------------
*--------------------------------- AUXILIAR MODELS -----------------------------
*-------------------------------------------------------------------------------

*-------------------------------------------------------------------------------
*--  Aux_SAM_Calibration = SAM disaggregation in load blocks and technologies --
*-------------------------------------------------------------------------------
$label Model_Aux_SAM_Calibration

*-----------ELECTRICITY MODEL EXECUTION
$goto Model_BU
$label Model_BU_Return_to_Aux_SAM_Calibration

*        sending optimization results as parameters to the following model
         parameters energy_only_price_SAMDIS(y,l,dp,db), pgen_SAMDIS(y,t,f,l,gp,gb), ppumped_SAMDIS(y,l,gp,gb), BU_mkt_surplus(y,l,p,b), BU_dist_factor(y,l,p,b), BU_market_failures_and_non_accounted_costs(y,l,p,b);

         Send_Results_LEGEN_BAU_to_SAMDIS


*-----------SAM DISAGGREGATION MODEL EXECUTION
$include ./Models/Aux_SAM_Calibration - SAM disaggregation in load blocks and technologies.gms
         if (exec_MinMax=1 or exec_MinMax_plus_lineal_term=1, SOLVE SAMDIS USING LP MINIMIZING OBJECTIVE_SAMDIS;execution_log_file(SAMDIS));
         if (exec_Quadratic=1, SOLVE NON_LINEAL_SAMDIS USING NLP MINIMIZING NON_LINEAL_OBJECTIVE_SAMDIS;execution_log_file(NON_LINEAL_SAMDIS));

*        exporting results
$if %excel_export%==1 $include ./Auxiliar_Output_Files/Aux_SAM_Calibration Output.gms

* exporting results
         if ( (%gdx_export%=1) and (exec_MinMax=1), execute_unload 'SAMDIS_MinMax.gdx');
         if ( (%gdx_export%=1) and (exec_MinMax_plus_lineal_term=1), execute_unload 'SAMDIS_MinMaxPlus.gdx');
         if ( (%gdx_export%=1) and (exec_Quadratic=1), execute_unload 'SAMDIS_Quadractic.gdx');

* Returning to GEMED Model
$If "%Model%" == "GEMED" $goto Model_Aux_SAM_Calibration_Return_to_GEMED
$If "%Model%" == "TAX_GEMED" $goto Model_Aux_SAM_Calibration_Return_to_TAX_GEMED

$goto end
*-------------------------------------------------------------------------------
*- Aux_SAM_Iter_Calibration = Iterative SAM disaggregation and electricity model results compatibility -
*-------------------------------------------------------------------------------
$label Model_Aux_SAM_Iter_Calibration

*-----------GENERAL OPTIONS to define maximun number of iterations and solution tolerance
         set     iter(iterations) max number of iterations        /1*50/
                 iter_executed(iterations);

         scalar  end_loop        auxiliar exit loop scalar       /0/
                 tol             percentage tolerance            /1e-8/;

*-----------INITIALIZING MODELS
*        ELECTRICITY MODEL
$include ./Models/BU - Bottom-up Electricity Model.gms

* fixing pumping to zero to simplify model debug
*PPUMPED_LEGEN.fx(eys,l,p,b)=0;

         if (exec_BU_MCP=1, SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP;);

*initializing parameters
parameters energy_only_price_SAMDIS(y,l,dp,db), pgen_SAMDIS(y,t,f,l,gp,gb), ppumped_SAMDIS(y,l,gp,gb), BU_mkt_surplus(y,l,p,b), BU_dist_factor(y,l,p,b), BU_market_failures_and_non_accounted_costs(y,l,p,b);
energy_only_price_SAMDIS(y,l,dp,db)=0; pgen_SAMDIS(y,t,f,l,gp,gb)=0; ppumped_SAMDIS(y,l,gp,gb)=0; BU_mkt_surplus(y,l,p,b)=0; BU_dist_factor(y,l,p,b)=0; BU_market_failures_and_non_accounted_costs(y,l,p,b)=0;

*-----------SAM DISAGGREGATION MODEL
$include ./Models/Aux_SAM_Calibration - SAM disaggregation in load blocks and technologies.gms

*----------- STARTING EXECUTIONS
         loop (iter $ ( (end_loop=0) and (iter.val>1) ) ,
                         iter_executed(iter)=yes;

* fixing pumping to zero to simplify model debug
*PPUMPED_LEGEN.fx(eys,l,p,b)=PPUMPED_LEGEN.l(eys,l,p,b);

*                        ELECTRICITY MODEL EXECUTION
                         if (exec_BU_LP=1, SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU));
                         if (exec_BU_MCP=1, SOLVE MCPGEN_BAU using MCP; execution_log_file(MCPGEN_BAU));

*                        sending optimization results as parameters to the following model
                         Send_Results_LEGEN_BAU_to_SAMDIS

*                        SAM DISAGGREGATION MODEL EXECUTION
                         if (exec_MinMax=1 or exec_MinMax_plus_lineal_term=1, SOLVE SAMDIS USING LP MINIMIZING OBJECTIVE_SAMDIS; execution_log_file(SAMDIS) );
                         if (exec_Quadratic=1, SOLVE NON_LINEAL_SAMDIS USING NLP MINIMIZING NON_LINEAL_OBJECTIVE_SAMDIS; execution_log_file(NON_LINEAL_SAMDIS) );

*                        reseting (initialising) parameters
                         Deviations_by_iteration

*                        Saving SAM disaggregation iteration results
                         Iteration_results

*                        sending optimization results as parameters to the following model
                         Send_Results_SAMDIS_to_LEGEN

*                        Exiting loop
                         if (exec_MinMax=1 or exec_MinMax_plus_lineal_term=1, if( (max_deviation(iter) < tol), end_loop=1 ); );
                         if (exec_Quadratic=1, if( (max(oem_vom_dev(iter),oem_fom_labor_dev(iter),oem_fom_social_contribution_dev(iter),oem_fom_equipments_dev(iter),thermodynamic_efficiency_dev(iter),own_cons_gen_pctg_dev(iter),overnight_costs_dev(iter),tdeo_losses_pctg_dev(iter),exp_price_pctg_dev(iter),imp_price_pctg_dev(iter),tdeo_pf_pct_adj_dev(iter),pollut_content_dev(iter)) < tol), end_loop=1 ););
         );

* exporting results
         if ( (%gdx_export%=1) and (exec_MinMax=1), execute_unload 'SAMDIS_MinMax.gdx');
         if ( (%gdx_export%=1) and (exec_MinMax_plus_lineal_term=1), execute_unload 'SAMDIS_MinMaxPlus.gdx');
         if ( (%gdx_export%=1) and (exec_Quadratic=1), execute_unload 'SAMDIS_Quadractic.gdx');
         if ( (%gdx_export%=1) and (exec_BU_LP=1), execute_unload 'LEGEN_BAU.gdx');
         if ( (%gdx_export%=1) and (exec_BU_MCP=1), execute_unload 'MCPGEN_BAU.gdx');

* exporting results
$if %excel_export%==1 $include ./Auxiliar_Output_Files/Aux_SAM_Iter_Calibration Output.gms

* returning to model 5 if this label was executed from there
$If "%Model%" == "HGEMED" $goto Model_Aux_SAM_Iter_Calibration_Return_HGEMED
$If "%Model%" == "TAX_HGEMED" $goto Model_Aux_SAM_Iter_Calibration_Return_TAX_HGEMED
$If "%Model%" == "DR_HGEMED" $goto Model_Aux_SAM_Iter_Calibration_Return_HGEMED

$goto end
*-------------------------------------------------------------------------------
*--- Aux_BU_Comparison = Bottom-up Electricity Lineal and MCP Results Comparison ---
*-------------------------------------------------------------------------------
$label Model_Aux_BU_Comparison

$include ./Models/BU - Bottom-up Electricity Model.gms

*PPUMPED_LEGEN.fx(eys,l,p,b)=0;
*INSTALLED_CAPACITY_LEGEN.lo(eys,l,t)=existent_installed_capacity(eys,l,t);

* start point solution
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU)

* exporting results
         if ( (%gdx_export%=1), execute_unload 'LEGEN_BAU.gdx');

*         OPTION ITERLIM = 0;

*PPUMPED_LEGEN.fx(eys,l,p,b)=PPUMPED_LEGEN.l(eys,l,p,b);

*INSTALLED_CAPACITY_LEGEN.lo(eys,l,t)=existent_installed_capacity(eys,l,t);
*RES_LEGEN.fx(eys,l,p1)=res_init(eys,l,"Hyd_Res");
*OPTION ITERLIM = 0;

*INSTALLED_CAPACITY_LEGEN.fx(eys,l,t)=INSTALLED_CAPACITY_LEGEN.l(eys,l,t);
* mcp model solution
         SOLVE MCPGEN_BAU using MCP; execution_log_file(MCPGEN_BAU)

         if ( (%gdx_export%=1), execute_unload 'MCPGEN_BAU.gdx');

parameter
EnergyOnlyPrice(eys,l,p,b),
DemandEquationMarginal(eys,l,p,b),
checkEnergyOnlyPrice(eys,l,p,b);

DemandEquationMarginal(eys,l,p,b)$duration(l,p,b)= eq_demand_balance_BAU.m(eys,l,p,b)/duration(l,p,b)*1e6;
EnergyOnlyPrice(eys,l,p,b)$duration(l,p,b)= - ( LAMBDA_DEM.l(eys,l,p,b)*1e6 ) / duration(l,p,b);
checkEnergyOnlyPrice(eys,l,p,b) = EnergyOnlyPrice(eys,l,p,b) - DemandEquationMarginal(eys,l,p,b);

display    EnergyOnlyPrice,  DemandEquationMarginal, checkEnergyOnlyPrice;


$ontext
FUEL_COST_LEGEN.l(eys,t,f,l,p,b) = PGEN_LEGEN.l(eys,t,f,l,p,b)*thermodynamic_efficiency(eys,t,f)*fuel_price(eys,t,f)/1e6;
CO2_PAYMENTS_LEGEN.l(eys,t,f,l,p,b) = PGEN_LEGEN.l(eys,t,f,l,p,b)*co2e_content(f,t)/1e6*co2_price(eys);
VOM_LEGEN.l(eys,t,f,l,p,b) = init_oem_vom(eys,t)*PGEN_LEGEN.l(eys,t,f,l,p,b)/1e6;
INSTALLED_CAPACITY_LEGEN.l(eys,l,t)= existent_installed_capacity(eys,l,t) + sum((eys_alias)$((ord(eys_alias) le ord(eys)) and (ord(eys_alias) gt (ord(eys)-power_plant_time_life(t)))),PINS_LEGEN.l(eys_alias,l,t));
AMORTIZATION_LEGEN.l(eys,l,t)= init_overnight_costs(eys,t) * IDC(t) * capital_recovery_factor(t) * ( existent_installed_capacity_to_be_amortized(eys,l,t) +  sum((eys_alias)$((ord(eys_alias) le ord(eys)) and (ord(eys_alias) gt (ord(eys)-power_plant_time_life(t)))),PINS_LEGEN.l(eys_alias,l,t)) )/1e3;
$offtext

$goto end


*-------------------------------------------------------------------------------
*------------------------------ POLICY EVALUATIONS -----------------------------
*-------------------------------------------------------------------------------



*-------------------------------------------------------------------------------
*--- DR_BU = Demand Response in the pure Bottom-up Electricity Expansion Lineal Model ---
*-------------------------------------------------------------------------------
$label Model_DR_BU

if( not(exec_DR_displac=1 or exec_DR_reduc=1), exec_DR_displac=1; exec_DR_reduc=1 );

*-----------MODEL EXECUTION

*        ELECTRICITY MODEL EXECUTION (Initial execution in order to define gad initial prices)
$include ./Models/BU - Bottom-up Electricity Model.gms
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU)

*        BAU results
         parameters energy_only_price_BAU(eys,l,p,b),energy_only_price_Case_study(eys,l,p,b);
         if (exec_BU_LP=1, energy_only_price_BAU(eys,l,p,b)$duration(l,p,b)= eq_demand_balance_BAU.m(eys,l,p,b)/duration(l,p,b)*1e6;);
         if (exec_BU_MCP=1, energy_only_price_BAU(eys,l,p,b)$duration(l,p,b)= - ( LAMBDA_DEM.l(eys,l,p,b)*1e6 ) / duration(l,p,b);  );
         LEGEN_results(BAU)

*        initial reference price to GAD
         GAD_energy_only_price(eys,l,p,b)$duration(l,p,b)=energy_only_price_BAU(eys,l,p,b);

*-----------FINAL EXECUTION
         SOLVE LEGEN minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN)

*        Case study results
         if (exec_BU_LP=1, energy_only_price_Case_study(eys,l,p,b)$duration(l,p,b)= eq_demand_balance_Case_study.m(eys,l,p,b)/duration(l,p,b)*1e6;);
         if (exec_BU_MCP=1, energy_only_price_Case_study(eys,l,p,b)$duration(l,p,b)= - ( LAMBDA_DEM.l(eys,l,p,b)*1e6 ) / duration(l,p,b);  );
         LEGEN_results(Case_study)

*        comparing results
         LEGEN_results_comparison

*        exporting results
$if %excel_export%==1 $include ./Auxiliar_Output_Files/BU Output.gms

*        output DR policy assessment results to excel
$if %excel_export%==1 $include ./Auxiliar_output_files/BU Comparison Output.gms


$goto end


*-------------------------------------------------------------------------------
*------------------ DR_CGE = Demand Response in the pure CGE Model -------------
*-------------------------------------------------------------------------------
$label Model_DR_CGE

if( not(exec_DR_displac=0 and exec_DR_reduc=1), exec_DR_displac=0; exec_DR_reduc=1 );

*-----------ELECTRICITY MODEL EXECUTION
$include ./Models/BU - Bottom-up Electricity Model.gms
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU)

*        initial reference price to GAD
         GAD_energy_only_price(eys,l,p,b)$duration(l,p,b)=eq_demand_balance_BAU.m(eys,l,p,b)/duration(l,p,b)*1e6;

*-----------CGE MODEL EXECUTION
$include ./Models/CGE - Top-down Pure CGE Model.gms
         if (exec_MCP=1, SOLVE CGE_MCP using MCP; execution_log_file(CGE_MCP));
         if (exec_NLP=1, SOLVE CGE_NLP minimizing DUMMY_CGE using nlp; execution_log_file(CGE_NLP));

*        returning parameters and variables to original non scaled values
         CGE_scale_parameters(*)
         CGE_scale_results(*)

*        BAU results
         CGE_results(BAU)

*-----------ELECTRICITY MODEL EXECUTION
         SOLVE LEGEN minimizing OBJECTIVE_LEGEN using LP; abort $ (LEGEN.modelstat > 2) "No optimal solution found";

         Q_H_D_CGE.fx(ey,"Electricity")= ( Q_H_D_CGE.l(ey,"Electricity") * ( 1 - ( sum((l,b,p),CONSERVED_DR_LOAD.l(ey,l,p,b)) / sum((l,b,p),demand(ey,"Households",l,p,b) ) ) ) ) / scale_factor;

*        scaling parameters
         CGE_scale_parameters(/)
         CGE_scale_results(/)

*-----------CASE STUDY EXECUTION
         if (exec_MCP=1, SOLVE CGE_MCP using MCP; execution_log_file(CGE_MCP));
         if (exec_NLP=1, SOLVE CGE_NLP minimizing DUMMY_CGE using nlp; execution_log_file(CGE_NLP));

*        returning parameters and variables to original non scaled values
         CGE_scale_parameters(*)
         CGE_scale_results(*)

*        case study results
         CGE_results(Case_study)

*        comparing results
         CGE_results_comparison

*        output results to excel
$if %excel_export%==1 $include ./Auxiliar_output_files/CGE Output.gms

$goto end
*-------------------------------------------------------------------------------
*------ DR_GEMED = Demand Response in the GEMED Model with single SAM calibration -------
*-------------------------------------------------------------------------------
$label Model_DR_GEMED

if( not(exec_DR_displac=1 or exec_DR_reduc=1), exec_DR_displac=1; exec_DR_reduc=1 );

*Makes more sense to use the MinMax calibration with only one calibration round
*exec_MinMax=0; exec_MinMax_plus_lineal_term=1; exec_Quadratic=0;

*-----------ELECTRICITY MODEL EXECUTION
$include ./Models/BU - Bottom-up Electricity Model.gms
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU)

*        sending optimization results as parameters to the following model
         parameters energy_only_price_SAMDIS(y,l,dp,db), pgen_SAMDIS(y,t,f,l,gp,gb), ppumped_SAMDIS(y,l,gp,gb), BU_mkt_surplus(y,l,p,b), BU_dist_factor(y,l,p,b), BU_market_failures_and_non_accounted_costs(y,l,p,b);
         Send_Results_LEGEN_BAU_to_SAMDIS

*        initial reference price to GAD
         GAD_energy_only_price(eys,l,p,b)$duration(l,p,b)=eq_demand_balance_BAU.m(eys,l,p,b)/duration(l,p,b)*1e6;

*        dumping results to gdx file
         execute_unload 'LEGENBAU_%Model%_%Load_Block_Aggregation%_%Calibration_Problem%_%Years_Simulated%.gdx';
*         execute 'gdx2xls LEGENBAU_%Model%_%Load_Block_Aggregation%_%Calibration_Problem%_%Years_Simulated%.gdx LEGENBAU_%Model%_%Load_Block_Aggregation%_%Calibration_Problem%_%Years_Simulated%.xml';

*-----------SAM DISAGGREGATION MODEL EXECUTION
$include ./Models/Aux_SAM_Calibration - SAM disaggregation in load blocks and technologies.gms
         if (exec_MinMax=1 or exec_MinMax_plus_lineal_term=1, SOLVE SAMDIS USING LP MINIMIZING OBJECTIVE_SAMDIS; execution_log_file(SAMDIS));
         if (exec_Quadratic=1, SOLVE NON_LINEAL_SAMDIS USING NLP MINIMIZING NON_LINEAL_OBJECTIVE_SAMDIS; execution_log_file(NON_LINEAL_SAMDIS));

*        sending optimization results as parameters to the following model
         Send_Results_SAMDIS_to_GEMED

*        dumping results to gdx file
         execute_unload 'SAMDIS_%Model%_%Load_Block_Aggregation%_%Calibration_Problem%_%Years_Simulated%.gdx';

*-----------EXTENDEND CGE MODEL EXECUTION
$include ./Models/GEMED - Electricity Extended Top-down Pure CGE Model.gms
         if (exec_MCP=1, SOLVE GEMED_MCP using MCP; execution_log_file(GEMED_MCP));
         if (exec_NLP=1, SOLVE GEMED_NLP minimizing DUMMY_GEMED using nlp; execution_log_file(GEMED_NLP));

*        returning parameters and variables to original non scaled values
         GEMED_scale_parameters(*)
         GEMED_scale_results(*)

*        BAU results
         parameters energy_only_price_BAU(eys,l,p,b),energy_only_price_Case_study(eys,l,p,b);
         if (exec_BU_LP=1, energy_only_price_BAU(eys,l,p,b)$duration(l,p,b)= eq_demand_balance_BAU.m(eys,l,p,b)/duration(l,p,b)*1e6;);
         if (exec_BU_MCP=1, energy_only_price_BAU(eys,l,p,b)$duration(l,p,b)= - ( LAMBDA_DEM.l(eys,l,p,b)*1e6 ) / duration(l,p,b);  );
         LEGEN_results(BAU)
         GEMED_CGE_results(BAU)
         DR_energy_economics_results(BAU)

*        dumping results to gdx file
         execute_unload 'GEMEDBAU_%Model%_%Load_Block_Aggregation%_%Calibration_Problem%_%Years_Simulated%.gdx';

*-----------CASE STUDY EXECUTION
*-----------ELECTRICITY MODEL EXECUTION
         SOLVE LEGEN minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN)

*        fixing DR policy consequences
         Q_H_D_GEN_GEMED.fx(ey,l,p,b) = ( Q_H_D_GEN_GEMED.l(ey,l,p,b) * ( 1 + ( ( (INCREASED_DR_LOAD.l(ey,l,p,b) - DECREASED_DR_LOAD.l(ey,l,p,b) )$(exec_DR_displac) - ( CONSERVED_DR_LOAD.l(ey,l,p,b) )$(exec_DR_reduc) ) / demand(ey,"Households",l,p,b) ) )) / scale_factor;

*        scale parameters
         GEMED_scale_parameters(/)
         GEMED_scale_results(/)

*        dumping results to gdx file
         execute_unload 'LEGEN_%Model%_%Load_Block_Aggregation%_%Calibration_Problem%_%Years_Simulated%.gdx';

*-----------GEMED MODEL EXECUTION
*SOLVE GEMED_MCP_GAD using MCP; execution_log_file(GEMED_MCP)

         if (exec_MCP=1, SOLVE GEMED_MCP using MCP; execution_log_file(GEMED_MCP));
         if (exec_NLP=1, SOLVE GEMED_NLP minimizing DUMMY_GEMED using nlp; execution_log_file(GEMED_NLP));

*        returning parameters and variables to original non scaled values
         GEMED_scale_parameters(*)
         GEMED_scale_results(*)

*        case study results
         if (exec_BU_LP=1, energy_only_price_Case_study(eys,l,p,b)$duration(l,p,b)= eq_demand_balance_Case_study.m(eys,l,p,b)/duration(l,p,b)*1e6;);
         if (exec_BU_MCP=1, energy_only_price_Case_study(eys,l,p,b)$duration(l,p,b)= - ( LAMBDA_DEM.l(eys,l,p,b)*1e6 ) / duration(l,p,b);  );
         LEGEN_results(Case_study)
         GEMED_CGE_results(Case_study)
         DR_energy_economics_results(Case_study)

*        comparing results
         LEGEN_results_comparison
         GEMED_CGE_results_comparison
         GEMED_CGE_emissions_results_comparison
         DR_energy_economics_results_comparison

*        output calibration results
$if %excel_export%==1 $include ./Auxiliar_Output_Files/Aux_SAM_Calibration Output.gms
*        output LEGEN, CGE and/or GEMED results to excel
$if %excel_export%==1 $include ./Auxiliar_output_files/GEMED output.gms
*        output DR policy assessment results to excel
*$if %excel_export%==1 $include ./Auxiliar_output_files/BU Comparison Output.gms
*        output DR policy assessment results to excel
$if %excel_export%==1 $include ./Auxiliar_output_files/Paper_energy_economics_DR_results_output.gms

*        dumping results to gdx file
         execute_unload 'GEMED_%Model%_%Load_Block_Aggregation%_%Calibration_Problem%_%Years_Simulated%.gdx';

$goto end
*-------------------------------------------------------------------------------
*------ DR_GEMED_Iter_Calibration = Demand Response in the GEMED Model with iterative calibration ------
*-------------------------------------------------------------------------------
$label Model_DR_GEMED_Iter_Calibration

if( not(exec_DR_displac=1 or exec_DR_reduc=1), exec_DR_displac=1; exec_DR_reduc=1 );

*-----------GENERAL OPTIONS to define maximun number of iterations and solution tolerance
         set     iter(iterations) max number of iterations        /1*50/
                 iter_executed(iterations);

         scalar  end_loop        auxiliar exit loop scalar       /0/
                 tol             percentage tolerance            /1e-14/;

*-----------STARTING EXECUTION

*-----------BUSINESS AS USUAL EXECUTION

*-----------FIRST ITERATION
         iter_executed("1")=yes;

*        ELECTRICITY MODEL EXECUTION
$include ./Models/BU - Bottom-up Electricity Model.gms
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU)

*        sending optimization results as parameters to the following model
         parameters energy_only_price_SAMDIS(y,l,dp,db), pgen_SAMDIS(y,t,f,l,gp,gb), ppumped_SAMDIS(y,l,gp,gb), BU_mkt_surplus(y,l,p,b), BU_dist_factor(y,l,p,b), BU_market_failures_and_non_accounted_costs(y,l,p,b);
         Send_Results_LEGEN_BAU_to_SAMDIS

*        SAM DISAGGREGATION MODEL EXECUTION
$include ./Models/Aux_SAM_Calibration - SAM disaggregation in load blocks and technologies.gms
         if (exec_MinMax=1 or exec_MinMax_plus_lineal_term=1, SOLVE SAMDIS USING LP MINIMIZING OBJECTIVE_SAMDIS; execution_log_file(SAMDIS) );
         if (exec_Quadratic=1, SOLVE NON_LINEAL_SAMDIS USING NLP MINIMIZING NON_LINEAL_OBJECTIVE_SAMDIS; execution_log_file(NON_LINEAL_SAMDIS) );

*        Reseting (initialising) parameters
         Deviations_by_iteration

*        Saving SAM disaggregation iteration results
         Iteration_results

*-----------REMAINING EXECUTIONS
         loop (iter $ ( (end_loop=0) and (iter.val>1) ) ,
                         iter_executed(iter)=yes;

*                        sending optimization results as parameters to the following model
                         Send_Results_SAMDIS_to_LEGEN

*                        ELECTRICITY MODEL EXECUTION
                         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN_BAU)

*                        sending optimization results as parameters to the following model
                         Send_Results_LEGEN_BAU_to_SAMDIS

*                        SAM DISAGGREGATION MODEL EXECUTION
                         if (exec_MinMax=1 or exec_MinMax_plus_lineal_term=1, SOLVE SAMDIS USING LP MINIMIZING OBJECTIVE_SAMDIS; execution_log_file(SAMDIS) );
                         if (exec_Quadratic=1, SOLVE NON_LINEAL_SAMDIS USING NLP MINIMIZING NON_LINEAL_OBJECTIVE_SAMDIS; execution_log_file(NON_LINEAL_SAMDIS) );

*                        reseting parameters
                         Deviations_by_iteration

*                        auxiliar parameters for results evaluation
                         Iteration_results

*                        exiting loop
                         if (exec_MinMax=1 or exec_MinMax_plus_lineal_term=1, if( (max_deviation(iter) < tol), end_loop=1 ); );
                         if (exec_Quadratic=1, if( (max(oem_vom_dev(iter),oem_fom_labor_dev(iter),oem_fom_social_contribution_dev(iter),oem_fom_equipments_dev(iter),thermodynamic_efficiency_dev(iter),own_cons_gen_pctg_dev(iter),overnight_costs_dev(iter),tdeo_losses_pctg_dev(iter),exp_price_pctg_dev(iter),imp_price_pctg_dev(iter),tdeo_pf_pct_adj_dev(iter),co2e_content_dev(iter)) < tol), end_loop=1 ););
         );


*        sending optimization results as parameters to the following model
         Send_Results_SAMDIS_to_LEGEN

*        ELECTRICITY MODEL EXECUTION (executing electricity model with final calibration solution)
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP ; execution_log_file(LEGEN_BAU)

*        initial reference price to GAD
         GAD_energy_only_price(eys,l,p,b)$duration(l,p,b)=eq_demand_balance_BAU.m(eys,l,p,b)/duration(l,p,b)*1e6;

*        sending optimization results as parameters to the following model
         Send_Results_SAMDIS_to_GEMED

*        EXTENDEND CGE MODEL EXECUTION
$include ./Models/GEMED - Electricity Extended Top-down Pure CGE Model.gms
*GEMED_MCP.tolinfeas = 1e-2;
*GEMED_NLP.tolinfeas = 1e-2;
         if (exec_MCP=1, SOLVE GEMED_MCP using MCP; execution_log_file(GEMED_MCP));
         if (exec_NLP=1, SOLVE GEMED_NLP minimizing DUMMY_GEMED using nlp; execution_log_file(GEMED_NLP));

*        returning parameters and variables to original non scaled values
         GEMED_scale_parameters(*)
         GEMED_scale_results(*)

*        BAU results
         parameters energy_only_price_BAU(eys,l,p,b),energy_only_price_Case_study(eys,l,p,b);
         if (exec_BU_LP=1, energy_only_price_BAU(eys,l,p,b)$duration(l,p,b)= eq_demand_balance_BAU.m(eys,l,p,b)/duration(l,p,b)*1e6;);
         if (exec_BU_MCP=1, energy_only_price_BAU(eys,l,p,b)$duration(l,p,b)= - ( LAMBDA_DEM.l(eys,l,p,b)*1e6 ) / duration(l,p,b);  );
         LEGEN_results(BAU)
         GEMED_CGE_results(BAU)

*-----------CASE STUDY EXECUTION
*        ELECTRICITY MODEL EXECUTION
         SOLVE LEGEN minimizing OBJECTIVE_LEGEN using LP; execution_log_file(LEGEN)

*        fixing DR policy consequences
         Q_H_D_GEN_GEMED.fx(ey,l,p,b) = ( Q_H_D_GEN_GEMED.l(ey,l,p,b) * ( 1 + ( ( (INCREASED_DR_LOAD.l(ey,l,p,b) - DECREASED_DR_LOAD.l(ey,l,p,b) )$(exec_DR_displac) - ( CONSERVED_DR_LOAD.l(ey,l,p,b) )$(exec_DR_reduc) ) / demand(ey,"Households",l,p,b) ) ) ) / scale_factor;

*        returning parameters and variables to original non scaled values
         GEMED_scale_parameters(/)
         GEMED_scale_results(/)

*-----------CASE STUDY GEMED MODEL EXECUTION
         if (exec_MCP=1, SOLVE GEMED_MCP using MCP; execution_log_file(GEMED_MCP));
         if (exec_NLP=1, SOLVE GEMED_NLP minimizing DUMMY_GEMED using nlp; execution_log_file(GEMED_NLP));

*        returning parameters and variables to original non scaled values
         GEMED_scale_parameters(*)
         GEMED_scale_results(*)

*        case study results
         if (exec_BU_LP=1, energy_only_price_Case_study(eys,l,p,b)$duration(l,p,b)= eq_demand_balance_Case_study.m(eys,l,p,b)/duration(l,p,b)*1e6;);
         if (exec_BU_MCP=1, energy_only_price_Case_study(eys,l,p,b)$duration(l,p,b)= - ( LAMBDA_DEM.l(eys,l,p,b)*1e6 ) / duration(l,p,b);  );
         LEGEN_results(Case_study)
         GEMED_CGE_results(Case_study)

*        comparing results
         LEGEN_results_comparison
         GEMED_CGE_results_comparison
         GEMED_CGE_emissions_results_comparison

*        output results to excel
$if %excel_export%==1 $include ./Auxiliar_output_files/GEMED output.gms
$if %excel_export%==1 $include ./Auxiliar_Output_Files/Aux_SAM_Calibration Output.gms
$if %excel_export%==1 $include ./Auxiliar_Output_Files/Aux_SAM_Iter_Calibration Output.gms

$goto end
*-------------------------------------------------------------------------------
*----------------- DR_HGEMED = Demand Response in the H-GEMED Model ------------
*-------------------------------------------------------------------------------
$label Model_DR_HGEMED


*-----------H-GEMED INITIAL MODEL EXECUTION
$goto Model_HGEMED
$label Model_Aux_SAM_Iter_Calibration_Return_DR_HGEMED

*        BAU results
        HGEMED_results(BAU)

*        exporting results
         if ( (%gdx_export%=1) and (exec_MCP=1), execute_unload 'HGEMED_MCP_BAU.gdx');
         if ( (%gdx_export%=1) and (exec_NLP=1), execute_unload 'HGEMED_NLP_BAU.gdx');


*-----------SIMULATION


*-----------END OF SIMULATION DEFINITION

*        scale parameters
         HGEMED_scale_parameters(/)
         HGEMED_scale_results(/)

*-----------HGEMED MODEL EXECUTION - CASE STUDY
         if (exec_MCP=1, SOLVE HGEMED_MCP using MCP; execution_log_file(HGEMED_MCP));
         if (exec_NLP=1, SOLVE HGEMED_NLP minimizing DUMMY_HGEMED using nlp; execution_log_file(HGEMED_NLP));

*        returning parameters and variables to original non scaled values
         HGEMED_scale_parameters(*)
         HGEMED_scale_results(*)

*        case study results
         HGEMED_results(Case_study)

*        comparing results
         HGEMED_results_comparison
*         HGEMED_emissions_results_comparison

*        exporting results
         if ( (%gdx_export%=1) and (exec_MCP=1), execute_unload 'HGEMED_MCP_CASE_STUDY.gdx');
         if ( (%gdx_export%=1) and (exec_NLP=1), execute_unload 'HGEMED_NLP_CASE_STUDY.gdx');

*        output LEGEN, CGE and/or GEMED results to excel
*$if %excel_export%==1 $include ./Auxiliar_output_files/HGEMED output.gms

$goto end
*-------------------------------------------------------------------------------














*-------------------------------------------------------------------------------
*--- Tax_BU = Tax simulation in the pure Bottom-up Electricity Expansion Lineal Model ---
*-------------------------------------------------------------------------------
$label Model_TAX_BU

*-----------MODEL EXECUTION
*        ELECTRICITY MODEL EXECUTION
$include ./Models/BU - Bottom-up Electricity Model.gms
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP;
*execution_log_file(LEGEN_BAU)

*        BAU results
         parameters energy_only_price_BAU(eys,l,p,b),energy_only_price_Case_study(eys,l,p,b);
         if (exec_BU_LP=1, energy_only_price_BAU(eys,l,p,b)$duration(l,p,b)= eq_demand_balance_BAU.m(eys,l,p,b)/duration(l,p,b)*1e6;);
         if (exec_BU_MCP=1, energy_only_price_BAU(eys,l,p,b)$duration(l,p,b)= - ( LAMBDA_DEM.l(eys,l,p,b)*1e6 ) / duration(l,p,b);  );
         LEGEN_results(BAU)

*-----------SIMULATION
*SOx - 1000 /t = 1e-3 millones/t ----  2000 /t = 1e-3 millones/t
         pollutant_price(eys,"SOx",s)= %SOX_tax% / 1e6;
         pollutant_price(eys,"SOx","Households")= %SOX_tax% / 1e6;

*NOx - 1000 /t = 1e-3 millones/t ----  2000 /t = 1e-3 millones/t
         pollutant_price(eys,"NOx",s)= %NOX_tax% / 1e6;
         pollutant_price(eys,"NOx","Households")= %NOX_tax% / 1e6;
*-----------END OF SIMULATION DEFINITION


*-----------FINAL EXECUTION
         SOLVE LEGEN_BAU minimizing OBJECTIVE_LEGEN using LP;
*execution_log_file(LEGEN)

*        Case study results
         if (exec_BU_LP=1, energy_only_price_Case_study(eys,l,p,b)$duration(l,p,b)= eq_demand_balance_BAU.m(eys,l,p,b)/duration(l,p,b)*1e6;);
         if (exec_BU_MCP=1, energy_only_price_Case_study(eys,l,p,b)$duration(l,p,b)= - ( LAMBDA_DEM.l(eys,l,p,b)*1e6 ) / duration(l,p,b);  );
         LEGEN_results(Case_study)

*        comparing results
         LEGEN_results_comparison

*        exporting results
$if %excel_export%==1 $include ./Auxiliar_Output_Files/BU Output.gms

*        output DR policy assessment results to excel
$if %excel_export%==1 $include ./Auxiliar_output_files/BU Comparison Output.gms


$goto end


*-------------------------------------------------------------------------------
*------------------ Tax_CGE = Tax simulation in the pure CGE Model -------------
*-------------------------------------------------------------------------------
$label Model_TAX_CGE

*-----------CGE MODEL EXECUTION
$include ./Models/CGE - Top-down Pure CGE Model.gms
         if (exec_MCP=1, SOLVE CGE_MCP using MCP; execution_log_file(CGE_MCP));
         if (exec_NLP=1, SOLVE CGE_NLP minimizing DUMMY_CGE using nlp; execution_log_file(CGE_NLP));

*        returning parameters and variables to original non scaled values
         CGE_scale_parameters(*)
         CGE_scale_results(*)

*        BAU results
         CGE_results(BAU)

*        exporting results
         if ( (%gdx_export%=1) and (exec_MCP=1), execute_unload 'CGE_MCP_BAU.gdx');
         if ( (%gdx_export%=1) and (exec_NLP=1), execute_unload 'CGE_NLP_BAU.gdx');

*-----------SIMULATION
*SOx - 1000 /t = 1e-3 millones/t ----  2000 /t = 1e-3 millones/t
         pollutant_price(eys,"SOx",s)= %SOX_tax% / 1e6;
         pollutant_price(eys,"SOx","Households")= %SOX_tax% / 1e6;

*NOx - 1000 /t = 1e-3 millones/t ----  2000 /t = 1e-3 millones/t
         pollutant_price(eys,"NOx",s)= %NOX_tax% / 1e6;
         pollutant_price(eys,"NOx","Households")= %NOX_tax% / 1e6;

*CO2e difusos
         pollutant_price(eys,"CO2e_non_ETS",sam)$((sameas(sam,"Manufactures")) or (sameas(sam,"Transport")) or (sameas(sam,"Other_Services")) or (sameas(sam,"Households")))= %CO2e_diffuse_sectors_tax% ;

* Acquire the renewable premium money through taxes on final products
         exec_renew_premium_tax_reform = %renewable_premium_tax%;
         if ( ((%renewable_premium_tax%=4) or (%renewable_premium_tax%=5) or (%renewable_premium_tax%=6)), renewable_premium_payments(ey) = renewable_premium_payments("2013"); );

* set to: none, social_contributions or lump_sum to leave the environmental tax income to the public administration, or to apply it on subsidies to diminishes social contributions taxes or in lump sum transfers to productive sectors
if ( ( %green_tax_reform% = 1 ), exec_social_contributions_enviromental_tax_reform = 1; exec_lump_sum_enviromental_tax_reform = 0;  );
if ( ( %green_tax_reform% = 2 ), exec_social_contributions_enviromental_tax_reform = 0; exec_lump_sum_enviromental_tax_reform = 1;  );



if ( ( %fuel_tax_simulation% = 1 ),
         fuel_tx_rate(ey,s,g) = ( ( fuel_consumption("gasoline",s)*fuel_tax("gasoline","Simulation1A")/1e6 + fuel_consumption("gasoil",s)*fuel_tax("gasoil","Simulation1A")/1e6 ) / ( Q_II_CGE.l(ey,g,s)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Oil-Nuclear")) +
                                ( ( fuel_consumption("natural_gas",s)*fuel_tax("natural_gas","Simulation1A")/1e6 ) / ( Q_II_CGE.l(ey,g,s)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Gas"))  +
                                ( ( fuel_consumption("electricity",s)*fuel_tax("electricity","Simulation1A")/1e6 ) / ( Q_II_CGE.l(ey,g,s)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Electricity"));
         fuel_tx_rate(ey,"Households",g) = ( ( fuel_consumption("gasoline","Households")*fuel_tax("gasoline","Simulation1A")/1e6 + fuel_consumption("gasoil","Households")*fuel_tax("gasoil","Simulation1A")/1e6 ) / ( Q_H_D_CGE.l(ey,g)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Oil-Nuclear")) +
                                ( ( fuel_consumption("natural_gas","Households")*fuel_tax("natural_gas","Simulation1A")/1e6 ) / ( Q_H_D_CGE.l(ey,g)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Gas"))  +
                                ( ( fuel_consumption("electricity","Households")*fuel_tax("electricity","Simulation1A")/1e6 ) / ( Q_H_D_CGE.l(ey,g)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Electricity"));
                           );

if ( ( %fuel_tax_simulation% = 2 ),
         fuel_tx_rate(ey,s,g) = ( ( fuel_consumption("gasoline",s)*fuel_tax("gasoline","Simulation1B")/1e6 + fuel_consumption("gasoil",s)*fuel_tax("gasoil","Simulation1B")/1e6 ) / ( Q_II_CGE.l(ey,g,s)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Oil-Nuclear")) +
                                ( ( fuel_consumption("natural_gas",s)*fuel_tax("natural_gas","Simulation1B")/1e6 ) / ( Q_II_CGE.l(ey,g,s)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Gas"))  +
                                ( ( fuel_consumption("electricity",s)*fuel_tax("electricity","Simulation1B")/1e6 ) / ( Q_II_CGE.l(ey,g,s)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Electricity"));
         fuel_tx_rate(ey,"Households",g) = ( ( fuel_consumption("gasoline","Households")*fuel_tax("gasoline","Simulation1B")/1e6 + fuel_consumption("gasoil","Households")*fuel_tax("gasoil","Simulation1B")/1e6 ) / ( Q_H_D_CGE.l(ey,g)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Oil-Nuclear")) +
                                ( ( fuel_consumption("natural_gas","Households")*fuel_tax("natural_gas","Simulation1B")/1e6 ) / ( Q_H_D_CGE.l(ey,g)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Gas"))  +
                                ( ( fuel_consumption("electricity","Households")*fuel_tax("electricity","Simulation1B")/1e6 ) / ( Q_H_D_CGE.l(ey,g)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Electricity"));
                           );


display fuel_tx_rate,fuel_consumption,fuel_tax,Q_II_CGE.l,P_Q_CGE.l;
*fuel_consumption(fuel_simulation,sam) = fuel_consumption_table("fuel_consumption",fuel_simulation,sam);
*fuel_tax(fuel_simulation,economics_for_energy_simulation) = fuel_tax_table("fuel_tax",fuel_simulation,economics_for_energy_simulation);


*-----------END OF SIMULATION DEFINITION

*        scale parameters
         CGE_scale_parameters(/)
         CGE_scale_results(/)

*-----------CASE STUDY EXECUTION
         if (exec_MCP=1, SOLVE CGE_MCP using MCP; execution_log_file(CGE_MCP));
         if (exec_NLP=1, SOLVE CGE_NLP minimizing DUMMY_CGE using nlp; execution_log_file(CGE_NLP));

*        returning parameters and variables to original non scaled values
         CGE_scale_parameters(*)
         CGE_scale_results(*)

*        case study results
         CGE_results(Case_study)

*        comparing results
         CGE_results_comparison

*        exporting results
         if ( (%gdx_export%=1) and (exec_MCP=1), execute_unload 'CGE_MCP_CASE_STUDY.gdx');
         if ( (%gdx_export%=1) and (exec_NLP=1), execute_unload 'CGE_NLP_CASE_STUDY.gdx');

*        output results to excel
$if %excel_export%==1 $include ./Auxiliar_output_files/CGE Output.gms

$goto end
*-------------------------------------------------------------------------------
*------------ Tax_GEMED = Tax simulation in the Electricity Extended Top-down Pure CGE Model -------------
*-------------------------------------------------------------------------------
$label Model_TAX_GEMED

*-----------SAM CALIBRATION MODEL EXECUTION
$goto Model_Aux_SAM_Calibration
$label Model_Aux_SAM_Calibration_Return_to_TAX_GEMED

*        sending optimization results as parameters to the following model
         Send_Results_SAMDIS_to_GEMED

*-----------GEMED MODEL EXECUTION - BAU
$include ./Models/GEMED - Electricity Extended Top-down Pure CGE Model.gms
         if (exec_MCP=1, SOLVE GEMED_MCP using MCP; execution_log_file(GEMED_MCP));
         if (exec_NLP=1, SOLVE GEMED_NLP minimizing DUMMY_GEMED using nlp; execution_log_file(GEMED_NLP));

*        returning parameters and variables to original non scaled values
         GEMED_scale_parameters(*)
         GEMED_scale_results(*)

*        BAU results
         GEMED_CGE_results(BAU)

*        exporting results
         if ( (%gdx_export%=1) and (exec_MCP=1), execute_unload 'GEMED_MCP_BAU.gdx');
         if ( (%gdx_export%=1) and (exec_NLP=1), execute_unload 'GEMED_NLP_BAU.gdx');

*-----------SIMULATION
*SOx - 1000 /t = 1e-3 millones/t ----  2000 /t = 1e-3 millones/t
         pollutant_price(eys,"SOx",s)= %SOX_tax% / 1e6;
         pollutant_price(eys,"SOx","Households")= %SOX_tax% / 1e6;

*NOx - 1000 /t = 1e-3 millones/t ----  2000 /t = 1e-3 millones/t
         pollutant_price(eys,"NOx",s)= %NOX_tax% / 1e6;
         pollutant_price(eys,"NOx","Households")= %NOX_tax% / 1e6;

*CO2e difusos
         pollutant_price(eys,"CO2e_non_ETS",sam)$((sameas(sam,"Manufactures")) or (sameas(sam,"Transport")) or (sameas(sam,"Other_Services")) or (sameas(sam,"Households")))= %CO2e_diffuse_sectors_tax% ;

* Acquire the renewable premium money through taxes on final products
         exec_renew_premium_tax_reform = %renewable_premium_tax%;
         if ( ((%renewable_premium_tax%=4) or (%renewable_premium_tax%=5) or (%renewable_premium_tax%=6)), renewable_premium_payments(ey) = renewable_premium_payments("2013"); );

* set to: none, social_contributions or lump_sum to leave the environmental tax income to the public administration, or to apply it on subsidies to diminishes social contributions taxes or in lump sum transfers to productive sectors
         if ( ( %green_tax_reform% = 1 ), exec_social_contributions_enviromental_tax_reform = 1; exec_lump_sum_enviromental_tax_reform = 0;  );
         if ( ( %green_tax_reform% = 2 ), exec_social_contributions_enviromental_tax_reform = 0; exec_lump_sum_enviromental_tax_reform = 1;  );


*Did not included fuel tax payments on generation and t&D sectors!!!!
if ( ( %fuel_tax_simulation% = 1 ),
         fuel_tx_rate(ey,sne,gne) = ( ( fuel_consumption("gasoline",sne)*fuel_tax("gasoline","Simulation1A")/1e6 + fuel_consumption("gasoil",sne)*fuel_tax("gasoil","Simulation1A")/1e6 ) / ( Q_II_GEMED.l(ey,gne,sne)*P_Q_GEMED.l(ey,gne) )  )$(sameas(gne,"Oil-Nuclear")) +
                                ( ( fuel_consumption("natural_gas",sne)*fuel_tax("natural_gas","Simulation1A")/1e6 ) / ( Q_II_GEMED.l(ey,gne,sne)*P_Q_GEMED.l(ey,gne) )  )$(sameas(gne,"Gas"));

         fuel_tx_rate(ey,sne,"Electricity") = ( fuel_consumption("electricity",sne)*fuel_tax("electricity","Simulation1A")/1e6 ) / ( sum((l,p,b),Q_II_GEN_SNE_GEMED.l(ey,l,p,b,sne)*P_Q_GEN_GEMED.l(ey,l,p,b)) );

         fuel_tx_rate(ey,"Households",gne) = ( ( fuel_consumption("gasoline","Households")*fuel_tax("gasoline","Simulation1A")/1e6 + fuel_consumption("gasoil","Households")*fuel_tax("gasoil","Simulation1A")/1e6 ) / ( Q_H_D_GEMED.l(ey,gne)*P_Q_GEMED.l(ey,gne) )  )$(sameas(gne,"Oil-Nuclear")) +
                                ( ( fuel_consumption("natural_gas","Households")*fuel_tax("natural_gas","Simulation1A")/1e6 ) / ( Q_H_D_GEMED.l(ey,gne)*P_Q_GEMED.l(ey,gne) ) )$(sameas(gne,"Gas"));
         fuel_tx_rate(ey,"Households","Electricity") = ( fuel_consumption("electricity","Households")*fuel_tax("electricity","Simulation1A")/1e6 ) / ( sum((l,p,b),Q_H_D_GEN_GEMED.l(ey,l,p,b)*P_Q_GEN_GEMED.l(ey,l,p,b)) ) ;
     );

if ( ( %fuel_tax_simulation% = 2 ),
         fuel_tx_rate(ey,sne,gne) = ( ( fuel_consumption("gasoline",sne)*fuel_tax("gasoline","Simulation1B")/1e6 + fuel_consumption("gasoil",sne)*fuel_tax("gasoil","Simulation1B")/1e6 ) / ( Q_II_GEMED.l(ey,gne,sne)*P_Q_GEMED.l(ey,gne) )  )$(sameas(gne,"Oil-Nuclear")) +
                                ( ( fuel_consumption("natural_gas",sne)*fuel_tax("natural_gas","Simulation1B")/1e6 ) / ( Q_II_GEMED.l(ey,gne,sne)*P_Q_GEMED.l(ey,gne) )  )$(sameas(gne,"Gas"));

         fuel_tx_rate(ey,sne,"Electricity") = ( fuel_consumption("electricity",sne)*fuel_tax("electricity","Simulation1B")/1e6 ) / ( sum((l,p,b),Q_II_GEN_SNE_GEMED.l(ey,l,p,b,sne)*P_Q_GEN_GEMED.l(ey,l,p,b)) );

         fuel_tx_rate(ey,"Households",gne) = ( ( fuel_consumption("gasoline","Households")*fuel_tax("gasoline","Simulation1B")/1e6 + fuel_consumption("gasoil","Households")*fuel_tax("gasoil","Simulation1B")/1e6 ) / ( Q_H_D_GEMED.l(ey,gne)*P_Q_GEMED.l(ey,gne) )  )$(sameas(gne,"Oil-Nuclear")) +
                                ( ( fuel_consumption("natural_gas","Households")*fuel_tax("natural_gas","Simulation1B")/1e6 ) / ( Q_H_D_GEMED.l(ey,gne)*P_Q_GEMED.l(ey,gne) ) )$(sameas(gne,"Gas"));
         fuel_tx_rate(ey,"Households","Electricity") = ( fuel_consumption("electricity","Households")*fuel_tax("electricity","Simulation1B")/1e6 ) / ( sum((l,p,b),Q_H_D_GEN_GEMED.l(ey,l,p,b)*P_Q_GEN_GEMED.l(ey,l,p,b)) ) ;
     );

*-----------END OF SIMULATION DEFINITION

*        scale parameters
         GEMED_scale_parameters(/)
         GEMED_scale_results(/)

*-----------GEMED MODEL EXECUTION - CASE STUDY
         if (exec_MCP=1, SOLVE GEMED_MCP using MCP; execution_log_file(GEMED_MCP));
         if (exec_NLP=1, SOLVE GEMED_NLP minimizing DUMMY_GEMED using nlp; execution_log_file(GEMED_NLP));

*        returning parameters and variables to original non scaled values
         GEMED_scale_parameters(*)
         GEMED_scale_results(*)

*        case study results
         GEMED_CGE_results(Case_study)

*        comparing results
         GEMED_CGE_results_comparison
         GEMED_CGE_emissions_results_comparison

*        exporting results
         if ( (%gdx_export%=1) and (exec_MCP=1), execute_unload 'GEMED_MCP_CASE_STUDY.gdx');
         if ( (%gdx_export%=1) and (exec_NLP=1), execute_unload 'GEMED_NLP_CASE_STUDY.gdx');

*        output LEGEN, CGE and/or GEMED results to excel
$if %excel_export%==1 $include ./Auxiliar_output_files/GEMED output.gms

$goto end


*-------------------------------------------------------------------------------
*------------ Tax_HGEMED = Tax simulation in the Electricity Extended Hybrid CGE Model -------------
*-------------------------------------------------------------------------------
$label Model_TAX_HGEMED

*-----------H-GEMED INITIAL MODEL EXECUTION
*Forcing BU model to be solved as MCP to provide the initial solution to the HGEMED model
exec_BU_MCP=1;
exec_BU_LP=0;

*-----------ITERATIVE CALIBRATION MODEL EXECUTION
$goto Model_Aux_SAM_Iter_Calibration
$label Model_Aux_SAM_Iter_Calibration_Return_TAX_HGEMED


*-----------EXECUTING MCP BU MODEL TO SERVE AS STARTING POINT
*-----------SETTING INSTALLED CAPACITY TO 2014 LEVELS TO APPROXIMATE 2014 ELECTRICITY MARKET BEHAVIOR
*INSTALLED_CAPACITY_HGEMED.lo("2005",l,t)=existent_installed_capacity("2014",l,t);
*INSTALLED_CAPACITY_LEGEN.up("2005",l,t)=existent_installed_capacity("2014",l,t);
*INSTALLED_CAPACITY_LEGEN.lo("2005",l,t)=existent_installed_capacity("2014",l,t);

         SOLVE MCPGEN_BAU using MCP;


*-----------H-GEMED MODEL EXECUTION
$include ./Models/H-GEMED - Hybrid Top-down CGE Bottom-up Electricity Model.gms

*PPUMPED_HGEMED.fx(eys,l,p,b)=PPUMPED_LEGEN.l(eys,l,p,b);

         if (exec_MCP=1, SOLVE HGEMED_MCP using MCP; execution_log_file(HGEMED_MCP));
         if (exec_NLP=1, SOLVE HGEMED_NLP minimizing DUMMY_HGEMED using nlp; execution_log_file(HGEMED_NLP));

*        returning parameters and variables to original non scaled values
         HGEMED_scale_parameters(*)
         HGEMED_scale_results(*)

*        BAU results
         HGEMED_results(BAU)

*        exporting results
         if ( (%gdx_export%=1) and (exec_MCP=1), execute_unload 'HGEMED_MCP_BAU.gdx');
         if ( (%gdx_export%=1) and (exec_NLP=1), execute_unload 'HGEMED_NLP_BAU.gdx');


*-----------SIMULATION
*SOx - 1000 /t = 1e-3 millones/t ----  2000 /t = 1e-3 millones/t
         pollutant_price(eys,"SOx",s)= %SOX_tax% / 1e6;
         pollutant_price(eys,"SOx","Households")= %SOX_tax% / 1e6;

*NOx - 1000 /t = 1e-3 millones/t ----  2000 /t = 1e-3 millones/t
         pollutant_price(eys,"NOx",s)= %NOX_tax% / 1e6;
         pollutant_price(eys,"NOx","Households")= %NOX_tax% / 1e6;

*CO2e difusos
         pollutant_price(eys,"CO2e_non_ETS",sam)$((sameas(sam,"Manufactures")) or (sameas(sam,"Transport")) or (sameas(sam,"Other_Services")) or (sameas(sam,"Households")))= %CO2e_diffuse_sectors_tax% ;

*CO2e ETS
*        if ( (%CO2e_diffuse_sectors_tax%=30), pollutant_price(eys,"CO2e_ETS",sam)$((sameas(sam,"Coal")) or (sameas(sam,"Oil-Nuclear")) or (sameas(sam,"Gas")) or (sameas(sam,"Electricity")))= 30 ; );

* Acquire the renewable premium money through taxes on final products
         exec_renew_premium_tax_reform = %renewable_premium_tax%;
         if ( ((%renewable_premium_tax%=4) or (%renewable_premium_tax%=5) or (%renewable_premium_tax%=6)), renewable_premium_payments(ey) = renewable_premium_payments("2013"); );

* set to: none, social_contributions or lump_sum to leave the environmental tax income to the public administration, or to apply it on subsidies to diminishes social contributions taxes or in lump sum transfers to productive sectors
         if ( ( %green_tax_reform% = 1 ), exec_social_contributions_enviromental_tax_reform = 1; exec_lump_sum_enviromental_tax_reform = 0;  );
         if ( ( %green_tax_reform% = 2 ), exec_social_contributions_enviromental_tax_reform = 0; exec_lump_sum_enviromental_tax_reform = 1;  );

$ontext
if ( ( %fuel_tax_simulation% = 1 ),
         fuel_tx_rate(ey,s,g) = ( ( fuel_consumption("gasoline",s)*fuel_tax("gasoline","Simulation1A")/1e6 + fuel_consumption("gasoil",s)*fuel_tax("gasoil","Simulation1A")/1e6 ) / ( Q_II_CGE.l(ey,g,s)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Oil-Nuclear")) +
                                ( ( fuel_consumption("natural_gas",s)*fuel_tax("natural_gas","Simulation1A")/1e6 ) / ( Q_II_CGE.l(ey,g,s)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Gas"))  +
                                ( ( fuel_consumption("electricity",s)*fuel_tax("electricity","Simulation1A")/1e6 ) / ( Q_II_CGE.l(ey,g,s)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Electricity"));
         fuel_tx_rate(ey,"Households",g) = ( ( fuel_consumption("gasoline","Households")*fuel_tax("gasoline","Simulation1A")/1e6 + fuel_consumption("gasoil","Households")*fuel_tax("gasoil","Simulation1A")/1e6 ) / ( Q_H_D_CGE.l(ey,g)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Oil-Nuclear")) +
                                ( ( fuel_consumption("natural_gas","Households")*fuel_tax("natural_gas","Simulation1A")/1e6 ) / ( Q_H_D_CGE.l(ey,g)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Gas"))  +
                                ( ( fuel_consumption("electricity","Households")*fuel_tax("electricity","Simulation1A")/1e6 ) / ( Q_H_D_CGE.l(ey,g)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Electricity"));
                              );

if ( ( %fuel_tax_simulation% = 2 ),
         fuel_tx_rate(ey,s,g) = ( ( fuel_consumption("gasoline",s)*fuel_tax("gasoline","Simulation1B")/1e6 + fuel_consumption("gasoil",s)*fuel_tax("gasoil","Simulation1B")/1e6 ) / ( Q_II_CGE.l(ey,g,s)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Oil-Nuclear")) +
                                ( ( fuel_consumption("natural_gas",s)*fuel_tax("natural_gas","Simulation1B")/1e6 ) / ( Q_II_CGE.l(ey,g,s)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Gas"))  +
                                ( ( fuel_consumption("electricity",s)*fuel_tax("electricity","Simulation1B")/1e6 ) / ( Q_II_CGE.l(ey,g,s)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Electricity"));
         fuel_tx_rate(ey,"Households",g) = ( ( fuel_consumption("gasoline","Households")*fuel_tax("gasoline","Simulation1B")/1e6 + fuel_consumption("gasoil","Households")*fuel_tax("gasoil","Simulation1B")/1e6 ) / ( Q_H_D_CGE.l(ey,g)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Oil-Nuclear")) +
                                ( ( fuel_consumption("natural_gas","Households")*fuel_tax("natural_gas","Simulation1B")/1e6 ) / ( Q_H_D_CGE.l(ey,g)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Gas"))  +
                                ( ( fuel_consumption("electricity","Households")*fuel_tax("electricity","Simulation1B")/1e6 ) / ( Q_H_D_CGE.l(ey,g)*P_Q_CGE.l(ey,g) )  )$(sameas(g,"Electricity"));
                           );
$offtext
*-----------END OF SIMULATION DEFINITION

*        scale parameters
         HGEMED_scale_parameters(/)
         HGEMED_scale_results(/)

*-----------HGEMED MODEL EXECUTION - CASE STUDY
         if (exec_MCP=1, SOLVE HGEMED_MCP using MCP; execution_log_file(HGEMED_MCP));
         if (exec_NLP=1, SOLVE HGEMED_NLP minimizing DUMMY_HGEMED using nlp; execution_log_file(HGEMED_NLP));

*        returning parameters and variables to original non scaled values
         HGEMED_scale_parameters(*)
         HGEMED_scale_results(*)

*        case study results
         HGEMED_results(Case_study)

*        comparing results
         HGEMED_results_comparison
*         HGEMED_emissions_results_comparison

*        exporting results
         if ( (%gdx_export%=1) and (exec_MCP=1), execute_unload 'HGEMED_MCP_CASE_STUDY.gdx');
         if ( (%gdx_export%=1) and (exec_NLP=1), execute_unload 'HGEMED_NLP_CASE_STUDY.gdx');

*        output LEGEN, CGE and/or GEMED results to excel
$if %excel_export%==1 $include ./Auxiliar_output_files/HGEMED output.gms

$goto end
*-------------------------------------------------------------------------------




$label end
